// to read properties of the electrolyte
PtrList<IOdictionary> electrolyteDicts(electrolyteRegions.size());

// field of conductivity...changes with local material
PtrList<volScalarField> sigmaEl(electrolyteRegions.size());

// needed for potential calculation
PtrList<volVectorField> jEl(electrolyteRegions.size());
PtrList<volVectorField> jdEl(electrolyteRegions.size());
PtrList<surfaceScalarField> jdSurfaceEl(electrolyteRegions.size());
PtrList<volScalarField> sourceTermEl(electrolyteRegions.size());

// fields for species transport simulation
PtrList<volScalarField> potentialEl(electrolyteRegions.size());
PtrList<volVectorField> gradPotentialEl(electrolyteRegions.size());
PtrList<surfaceScalarField> gradPotentialSurfaceEl(electrolyteRegions.size());
PtrList<speciesMixture> speciesMixtureEl(electrolyteRegions.size());
PtrList<surfaceScalarField> psiEl(electrolyteRegions.size());

forAll(electrolyteRegions, i)
{
    electrolyteDicts.set
    (
    	i,
   	new IOdictionary
   	(
   	    IOobject
    	    (
     		"transportProperties",
     		runTime.constant(),
     		electrolyteRegions[i],
     		IOobject::MUST_READ,
     		IOobject::NO_WRITE
    	    )
   	)
    );
      
    sigmaEl.set
    (
        i,
        new volScalarField
        (
            IOobject
            (
                "sigma",
                runTime.timeName(),
                electrolyteRegions[i],
                IOobject::NO_READ,
                IOobject::NO_WRITE
            ),
            electrolyteRegions[i],
            dimensionedScalar("zero", dimensionSet(-1,-3,3,0,0,2,0), 0),
            zeroGradientFvPatchField<scalar>::typeName // same conductivity at patch as inside the cell
        )
    );
    
    jEl.set
    (
        i,
        new volVectorField
        (
            IOobject
            (
                "J",
                runTime.timeName(),
                electrolyteRegions[i],
                IOobject::NO_READ,
                IOobject::AUTO_WRITE
            ),
            electrolyteRegions[i],
            dimensionedVector("zero", dimensionSet(0,-2,0,0,0,1,0), vector(0,0,0))
        )
    );
    
    
    jdEl.set
    (
        i,
        new volVectorField
        (
            IOobject
            (
                "Jd",
                runTime.timeName(),
                electrolyteRegions[i],
                IOobject::NO_READ,
                IOobject::AUTO_WRITE
            ),
            electrolyteRegions[i],
            dimensionedVector("zero", dimensionSet(0,-2,0,0,0,1,0), vector(0,0,0)),
            fixedGradientFvPatchField<vector>::typeName // same jd at patch as inside the cell
        )
    );
    
    
    jdSurfaceEl.set
    (
        i,
        new surfaceScalarField
        (
            IOobject
            (
                "jdSurface",
                runTime.timeName(),
                electrolyteRegions[i],
                IOobject::NO_READ,
                IOobject::NO_WRITE
            ),
            electrolyteRegions[i],
            dimensionedScalar("zero", dimensionSet(0,-2,0,0,0,1,0), 0.0)
        )
    );   
    
    potentialEl.set
    (
        i,
        new volScalarField
        (
            IOobject
            (
                "potential",
                runTime.timeName(),
                electrolyteRegions[i],
                IOobject::NO_READ,
                IOobject::AUTO_WRITE
            ),
            electrolyteRegions[i],
            dimensionedScalar("potential",dimensionSet(1,2,-3,0,0,-1,0),0)
        )
    );
  
    gradPotentialEl.set
    (
        i,
        new volVectorField
        (
            IOobject
            (
                "gradPotential",
                runTime.timeName(),
                electrolyteRegions[i],
                IOobject::NO_READ,
                IOobject::AUTO_WRITE
            ),
            electrolyteRegions[i],
            dimensionedVector("gradPotential",dimensionSet(1,1,-3,0,0,-1,0),vector::zero)
        )
    );
    
    gradPotentialSurfaceEl.set
    (
        i,
        new surfaceScalarField
        (
            IOobject
            (
                "gradPotentialSurface",
                runTime.timeName(),
                electrolyteRegions[i],
                IOobject::NO_READ,
                IOobject::AUTO_WRITE
            ),
            electrolyteRegions[i],
            dimensionedScalar("zero", dimensionSet(1,1,-3,0,0,-1,0), 0.0)
        )
    ); 
    
    sourceTermEl.set
    (
        i,
        new volScalarField
        (
            IOobject
            (
                "sourceTerm",
                runTime.timeName(),
                electrolyteRegions[i],
                IOobject::NO_READ,
                IOobject::AUTO_WRITE
            ),
            electrolyteRegions[i],
            dimensionedScalar("sourceTerm",dimensionSet(0,-3,0,0,0,1,0),0)
        )
    );
    
    psiEl.set
    (
    	i,
    	new surfaceScalarField
    	(
            IOobject
            (
              "psi",
               runTime.timeName(),
               electrolyteRegions[i],
               IOobject::NO_READ,
               IOobject::AUTO_WRITE
             ),
             electrolyteRegions[i],
             dimensionedScalar("psi",dimensionSet(0,3,-1,0,0,0,0),0)
        )
    );                   
                                
    Info<< "Reading species ...";
    
    speciesMixtureEl.set 
    (
    	i, 
    	new speciesMixture 
    	(
    	  electrolyteRegions[i]
    	)
    ); 
      
    PtrList<specie>& speciesEl = speciesMixtureEl[i].species();
    
    Info<< "electrolyte contains "<< speciesEl.size() << " species..." << nl << endl;
    
    forAll(speciesEl, is)
    {
    	Info << speciesEl[is].name() << endl;
    }
     
    Info<< "... finished reading species" << endl;
    
    Info<< "Created electrolyte fields for region " << electrolytesNames[i]
        << " for time = " << runTime.timeName() << nl << endl;
}
