PtrList<IOdictionary> fluidDict(fluidRegions.size()); // for reading fluid properties

PtrList<dimensionedScalar> sigmaFluids(fluidRegions.size()); // conductivity

// needed for Faraday BC
PtrList<volVectorField> jFluids(fluidRegions.size());

// fields for concentration simulation
PtrList<dimensionedScalar> DFluids(fluidRegions.size());
PtrList<volScalarField> cFluids(fluidRegions.size());
PtrList<volScalarField> xFluids(fluidRegions.size());


forAll(fluidRegions, i)
{
    fluidDict.set
    (
    	i,
   	new IOdictionary
   	(
   	    IOobject
    	    (
     		"transportProperties",
     		runTime.constant(),
     		fluidRegions[i],
     		IOobject::MUST_READ_IF_MODIFIED,
     		IOobject::NO_WRITE
    	    )
   	)
    );

    sigmaFluids.set
    (
   	i,
   	new dimensionedScalar
	(
	    "sigma",
	    dimensionSet(-1,-3,3,0,0,2,0),
	    fluidDict[i]
	)
    );
		
    if(fluidRegions[i].name() == "Bi")
    {
    cFluids.set
    (
       	i,
       	new volScalarField
       	(
            IOobject
            (
                "c",
                runTime.timeName(),
                fluidRegions[i],
                IOobject::MUST_READ,
                IOobject::AUTO_WRITE
            ),
            fluidRegions[i]
       	)
    );

    xFluids.set
    (
        i,
        new volScalarField
        (
            IOobject
            (
                "x",
                runTime.timeName(),
                fluidRegions[i],
                IOobject::NO_READ,
                IOobject::AUTO_WRITE
            ),
            fluidRegions[i],
            dimensionedScalar("x",dimless,0.0)
        )
    );

    DFluids.set
    (
        i,
        new dimensionedScalar
        (
	    "D",
	    dimensionSet(0,2,-1,0,0,0,0),
            fluidDict[i]
        )
    );
    }
    
    jFluids.set
    (
        i,
        new volVectorField
        (
            IOobject
            (
                "J",
                runTime.timeName(),
                fluidRegions[i],
                IOobject::NO_READ,
                IOobject::AUTO_WRITE
            ),
            fluidRegions[i],
            dimensionedVector("zero", dimensionSet(0,-2,0,0,0,1,0), vector(0,0,0))
        )
    );
    
    Info<< "Created fluid fields for region " << fluidsNames[i]
        << " for time = " << runTime.timeName() << nl << endl;
}
