/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | www.openfoam.com
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "specie.H"

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::specie::specie
(
    const fvMesh& mesh,
    const word& specieName,
    const dictionary& specieDict
)
:
    volScalarField
    (
        IOobject
        (
            IOobject::groupName("c", specieName),
            mesh.time().timeName(),
            mesh,
            IOobject::MUST_READ,
            IOobject::AUTO_WRITE
        ),
        mesh
    ),
    name_(specieName),
    mesh_(mesh),
    specieDict_(specieDict),
    D_("D", dimensionSet(0,2,-1,0,0,0,0), specieDict_),
    nu_("nu", dimensionSet(0,0,0,0,0,0,0), specieDict_),
    active_(readBool(specieDict_.lookup("active")))
{}

// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

Foam::autoPtr<Foam::specie> Foam::specie::clone() const
{
    NotImplemented;
    return autoPtr<specie>(nullptr);
}

// ************************************************************************* //
