/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | www.openfoam.com
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Description
    This boundary condition provides a flux boundary condition to set a
    concentration gradient of a single electrolyte species according to 
    a current density.

Developed by
    Carolina Duczek

\*---------------------------------------------------------------------------*/

#ifndef speciesConcentrationFvPatchScalarFields_H
#define speciesConcentrationFvPatchScalarFields_H

#include "fvPatchFields.H"
#include "fixedGradientFvPatchFields.H"
#include "speciesMixture.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
             Class speciesConcentrationFvPatchScalarField Declaration
\*---------------------------------------------------------------------------*/

class speciesConcentrationFvPatchScalarField
:
    public fixedGradientFvPatchScalarField
{
    // molar mass ... if we work with mass concentration
    scalar molarMass_;
    Field<scalar> concentration_;
       
public:

    //- Runtime type information
    TypeName("speciesConcentration");


    // Constructors

        //- Construct from patch and internal field
        speciesConcentrationFvPatchScalarField
        (
            const fvPatch&,
            const DimensionedField<scalar, volMesh>&
        );
	
        //- Construct from patch, internal field and dictionary
        speciesConcentrationFvPatchScalarField
        (
            const fvPatch&,
            const DimensionedField<scalar, volMesh>&,
            const dictionary&
        );

        //- Construct by mapping given speciesConcentrationFvPatchScalarField onto
        //  a new patch
        speciesConcentrationFvPatchScalarField
        (
            const speciesConcentrationFvPatchScalarField&,
            const fvPatch&,
            const DimensionedField<scalar, volMesh>&,
            const fvPatchFieldMapper&
        );

        //- Construct as copy
        speciesConcentrationFvPatchScalarField
        (
            const speciesConcentrationFvPatchScalarField&
        );

        //- Construct and return a clone
        virtual tmp<fvPatchScalarField> clone() const
        {
            return tmp<fvPatchScalarField>
            (
                new speciesConcentrationFvPatchScalarField(*this)
            );
        }

        //- Construct as copy setting internal field reference
        speciesConcentrationFvPatchScalarField
        (
            const speciesConcentrationFvPatchScalarField&,
            const DimensionedField<scalar, volMesh>&
        );

        //- Construct and return a clone setting internal field reference
        virtual tmp<fvPatchScalarField> clone
        (
            const DimensionedField<scalar, volMesh>& iF
        ) const
        {
            return tmp<fvPatchScalarField>
            (
                new speciesConcentrationFvPatchScalarField(*this, iF)
            );
        }


    // Member functions
    	
    	//- Return the "concentration value" at the patch
        virtual tmp<Field<scalar>> value() const;

        //- Update the patch pressure gradient field
        virtual void updateCoeffs();

        //- Write
        virtual void write(Ostream&) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#include "volFields.H"
#endif
