/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | www.openfoam.com
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Description
    Class which collects properties of a single specie.

Developed by
    Carolina Duczek and Norbert Weber

\*---------------------------------------------------------------------------*/

#ifndef specie_H
#define specie_H

#include "dimensionedScalar.H"
#include "volFields.H"
#include "dictionaryEntry.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
                           Class specie Declaration
\*---------------------------------------------------------------------------*/

class specie
:
    public volScalarField
{
    // Private data

        word name_;

        const fvMesh& mesh_;
        const dictionary specieDict_;
        const dimensionedScalar D_;
        const dimensionedScalar nu_;
        const bool active_;
        

public:
    // Constructors

        //- Construct from components
        specie
        (
            const fvMesh& mesh,
            const word& name,
            const dictionary& specieDict
        );

        //- Return clone
        autoPtr<specie> clone() const;

        //- Return a pointer to a new specie created on freestore
        //  from Istream
        class iNew
        {
	    const fvMesh& mesh_;
        public:

            iNew
            (
		const fvMesh& mesh
            )
	    :
		mesh_(mesh)
            {}

            autoPtr<specie> operator()(Istream& is) const
            {
                dictionaryEntry ent(dictionary::null, is);
                return autoPtr<specie>(new specie(mesh_,ent.keyword(), ent));
            }
        };

    // Member Functions

        const word& name() const
        {
            return name_;
        }

        const word& keyword() const
        {
            return name();
        }

        const fvMesh& mesh() const
        {
	    return mesh_;
        }

        const dimensionedScalar& D() const
        {
            return D_;
        }

        const dimensionedScalar& nu() const
        {
            return nu_;
        }

        bool active() const
        {
            return active_;
        }
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
