// solves the species equations

Info<< "start calculating species equation..." <<endl; 

for (int corr=0; corr<nCorr; corr++)
{
    forAll(electrolyteRegions, i)
    {
        if (finalIter)
        {
            electrolyteRegions[i].data::add("finalIteration", true);
        }
        
        electrolyteMesh[i].map(jGlobal, jEl[i]);
        electrolyteMesh[i].map(potentialGlobal, potentialEl[i]);
        volScalarField& sigmaSalt = sigmaEl[i];
        volVectorField& jSalt = jEl[i];
        speciesMixture& speciesMix = speciesMixtureEl[i];
        surfaceScalarField& jdSurfaceSalt = jdSurfaceEl[i];
        
        volScalarField nuc=speciesMix.species()[0]*0;
        surfaceScalarField& psi = psiEl[i];
        
        // compute gradient of potential to calculate Phi
        // mapping the potential from the global mesh leads to the problem 
        // that the migration term is not divergence free if the simplest test case is used
        // therefore an additional calculation for potential gradient is necessary

    	//set sigmaSalt2 and nuFDgradc to zero since it is a sum
    	volVectorField nuFDgradc = jSalt * 0;
    	surfaceScalarField nuFDgradcS = jdSurfaceSalt * 0;
    	
    	forAll(speciesMix.species(), is)
   	{   	
    	    volScalarField& c = speciesMix.species()[is];
    	    const dimensionedScalar& nu = speciesMix.species()[is].nu();
    	    const dimensionedScalar& D = speciesMix.species()[is].D();
    	    const dimensionedScalar mu("mu", dimensionSet(-1,0,1,0,1,0,0), D.value()/R.value()/T0.value());
    	
    	    nuFDgradc += F*nu*D*fvc::grad(c);
    	    nuFDgradcS += F*nu*D*fvc::snGrad(c);
    	}
    	
        // solve species transport equation for n-1 species
        Foam::label is;
        for(is=0; is<speciesMix.species().size()-1; is++) 
        {
            electrolyteMesh[i].map(potentialGlobal, potentialEl[i]);
            volScalarField& potentialSalt = potentialEl[i];
            
            electrolyteMesh[i].map(gradPotentialGlobal, gradPotentialEl[i]);
            
            volScalarField& c = speciesMix.species()[is];
            const dimensionedScalar& nu = speciesMix.species()[is].nu();
            const dimensionedScalar& D = speciesMix.species()[is].D();
            const dimensionedScalar mu("mu", dimensionSet(-1,0,1,0,1,0,0), D.value()/R.value()/T0.value());
            
            Info<< "minimal c before solving species-Eq " << Foam::gMin(c) << endl;
            
            psi = fvc::interpolate(-nu * mu * F * ((jSalt + nuFDgradc) / sigmaSalt)) & electrolyteRegions[i].Sf();

            // solve concentration
            // the migration term is expressed flux based
            // earlier investigations of Weber and Mengyuan have shown that this approach is necessary to keep symmetry
            
            Info<< "c of " << speciesMix.species()[is].name() << " at interface electrolyte anode is " 
                << c.boundaryField()[electrolyteRegions[i].boundaryMesh().findPatchID("salt_to_Li")]*1 << " mol/m^3" << endl;
            Info<< "c of " << speciesMix.species()[is].name() << " at interface electrolyte cathode is " 
                << c.boundaryField()[electrolyteRegions[i].boundaryMesh().findPatchID("salt_to_Bi")]*1 << " mol/m^3" << endl;
                
            for (int nonOrth=0; nonOrth<=nNonOrthCorr; nonOrth++)
            {
                fvScalarMatrix speciesEqn
                (
                    fvm::ddt(c) - fvm::laplacian(D,c)
                );
                speciesEqn -= fvm::div(psi,c);

                speciesEqn.solve
                (
                    electrolyteRegions[i].solver
                    (
                        c.select
                        (
                            oCorr == nOuterCorr-1
                         && corr == nCorr-1
                         && nonOrth == nNonOrthCorr    
                        )
                    )
                );
            }

            c.correctBoundaryConditions();
            
            Info<< "solved species transport for " << c.name() << endl;
            Info<< "minimal c after solving species-Eq " << Foam::gMin(c) << endl;
            
            // sum up nu*c for the species
            // this is necessary for the electroneutrality condition
            nuc += nu*c;
        }
        
        // use condition for electroneutrality to compute the last species
        volScalarField& cs=speciesMix.species()[is]; 
        cs=-nuc/speciesMix.species()[is].nu();

        Info<< "computed species transport for " << cs.name() << endl;
        
        forAll(speciesMix.species(), is)
    	{   	
    	    volScalarField& c = speciesMix.species()[is];
       	    const dimensionedScalar& D = speciesMix.species()[is].D();
    	
    	    volVectorField gradC
	    (
    	        IOobject
    	        (
        	    IOobject::groupName("gradC", speciesMix.species()[is].name()),
        	    runTime.timeName(),
        	    electrolyteRegions[i],
        	    IOobject::NO_READ,
        	    IOobject::NO_WRITE
    	        ),
       	        electrolyteRegions[i],
    	        dimensionedVector("zero", dimensionSet(0,-4,0,0,1,0,0), vector::zero)
	    );

    	    gradC = fvc::grad(c);
    	
    	    // migrational term in species equation
    	    volScalarField cmt
    	    (
    	        IOobject
    	        (
        	    IOobject::groupName("cmt", speciesMix.species()[is].name()),
        	    runTime.timeName(),
        	    electrolyteRegions[i],
        	    IOobject::NO_READ,
        	    IOobject::NO_WRITE
    	        ),
    	        electrolyteRegions[i],
    	        dimensionedScalar("zero",dimensionSet(0,-1,-1,0,1,0,0),0)
    	    );
    	
    	    cmt = fvc::div(psi*electrolyteRegions[i].magSf(),c);
    	
    	    // diffusional term in species equation
    	    volScalarField cdt
    	    (
    	        IOobject
    	        (
        	    IOobject::groupName("cdt", speciesMix.species()[is].name()),
        	    runTime.timeName(),
        	    electrolyteRegions[i],
        	    IOobject::NO_READ,
        	    IOobject::NO_WRITE
    	        ),
    	        electrolyteRegions[i],
    	        dimensionedScalar("zero",dimensionSet(0,-3,-1,0,1,0,0),0)
    	    );
    	
    	    cdt = fvc::laplacian(D,c);

    	    gradC.write(); 
    	    cmt.write(); 
    	    cdt.write(); 
        }
        
        if (finalIter)
        {
            electrolyteRegions[i].data::remove("finalIteration");
        }
    }
}

Info<< "stop calculating species equation" << endl; 
